@@ -1,4 +1,5 @@
 use std::borrow::Borrow;
+use std::cmp;
 use std::fmt;
 use std::hash::{Hash, Hasher};
 use std::mem;
@@ -11,8 +12,12 @@ use std::slice;
 use array::{Array, ArrayExt};
 use array::Index;
 use CapacityError;
+use errors::PubCrateNew;
 use odds::char::encode_utf8;
 
+#[cfg(feature="serde-1")]
+use serde::{Serialize, Deserialize, Serializer, Deserializer};
+
 /// A string with a fixed capacity.
 ///
 /// The `ArrayString` is a string backed by a fixed size array. It keeps track
@@ -64,7 +69,7 @@ impl<A: Array<Item=u8>> ArrayString<A> {
     /// ```
     pub fn from(s: &str) -> Result<Self, CapacityError<&str>> {
         let mut arraystr = Self::new();
-        try!(arraystr.push_str(s));
+        arraystr.try_push_str(s)?;
         Ok(arraystr)
     }
 
@@ -80,7 +85,7 @@ impl<A: Array<Item=u8>> ArrayString<A> {
     pub fn from_byte_string(b: &A) -> Result<Self, Utf8Error> {
         let mut arraystr = Self::new();
         let s = try!(str::from_utf8(b.as_slice()));
-        let _result = arraystr.push_str(s);
+        let _result = arraystr.try_push_str(s);
         debug_assert!(_result.is_ok());
         Ok(arraystr)
     }
@@ -108,6 +113,26 @@ impl<A: Array<Item=u8>> ArrayString<A> {
     /// ```
     pub fn is_full(&self) -> bool { self.len() == self.capacity() }
 
+    /// Adds the given char to the end of the string.
+    ///
+    /// Returns `Ok` if the push succeeds.
+    ///
+    /// ***Panics*** if the backing array is not large enough to fit the additional char.
+    ///
+    /// ```
+    /// use arrayvec::ArrayString;
+    ///
+    /// let mut string = ArrayString::<[_; 2]>::new();
+    ///
+    /// string.push('a');
+    /// string.push('b');
+    ///
+    /// assert_eq!(&string[..], "ab");
+    /// ```
+    pub fn push(&mut self, c: char) {
+        self.try_push(c).unwrap();
+    }
+
     /// Adds the given char to the end of the string.
     ///
     /// Returns `Ok` if the push succeeds.
@@ -119,14 +144,14 @@ impl<A: Array<Item=u8>> ArrayString<A> {
     ///
     /// let mut string = ArrayString::<[_; 2]>::new();
     ///
-    /// string.push('a').unwrap();
-    /// string.push('b').unwrap();
-    /// let overflow = string.push('c');
+    /// string.try_push('a').unwrap();
+    /// string.try_push('b').unwrap();
+    /// let overflow = string.try_push('c');
     ///
     /// assert_eq!(&string[..], "ab");
     /// assert_eq!(overflow.unwrap_err().element(), 'c');
     /// ```
-    pub fn push(&mut self, c: char) -> Result<(), CapacityError<char>> {
+    pub fn try_push(&mut self, c: char) -> Result<(), CapacityError<char>> {
         let len = self.len();
         unsafe {
             match encode_utf8(c, &mut self.raw_mut_bytes()[len..]) {
@@ -139,6 +164,26 @@ impl<A: Array<Item=u8>> ArrayString<A> {
         }
     }
 
+    /// Adds the given string slice to the end of the string.
+    ///
+    /// Returns `Ok` if the push succeeds.
+    ///
+    /// ***Panics*** if the backing array is not large enough to fit the string.
+    ///
+    /// ```
+    /// use arrayvec::ArrayString;
+    ///
+    /// let mut string = ArrayString::<[_; 2]>::new();
+    ///
+    /// string.push_str("a");
+    /// string.push_str("d");
+    ///
+    /// assert_eq!(&string[..], "ad");
+    /// ```
+    pub fn push_str(&mut self, s: &str) {
+        self.try_push_str(s).unwrap()
+    }
+
     /// Adds the given string slice to the end of the string.
     ///
     /// Returns `Ok` if the push succeeds.
@@ -150,16 +195,16 @@ impl<A: Array<Item=u8>> ArrayString<A> {
     ///
     /// let mut string = ArrayString::<[_; 2]>::new();
     ///
-    /// string.push_str("a").unwrap();
-    /// let overflow1 = string.push_str("bc");
-    /// string.push_str("d").unwrap();
-    /// let overflow2 = string.push_str("ef");
+    /// string.try_push_str("a").unwrap();
+    /// let overflow1 = string.try_push_str("bc");
+    /// string.try_push_str("d").unwrap();
+    /// let overflow2 = string.try_push_str("ef");
     ///
     /// assert_eq!(&string[..], "ad");
     /// assert_eq!(overflow1.unwrap_err().element(), "bc");
     /// assert_eq!(overflow2.unwrap_err().element(), "ef");
     /// ```
-    pub fn push_str<'a>(&mut self, s: &'a str) -> Result<(), CapacityError<&'a str>> {
+    pub fn try_push_str<'a>(&mut self, s: &'a str) -> Result<(), CapacityError<&'a str>> {
         if s.len() > self.capacity() - self.len() {
             return Err(CapacityError::new(s));
         }
@@ -270,10 +315,11 @@ impl<A: Array<Item=u8>> fmt::Display for ArrayString<A> {
 /// `Write` appends written data to the end of the string.
 impl<A: Array<Item=u8>> fmt::Write for ArrayString<A> {
     fn write_char(&mut self, c: char) -> fmt::Result {
-        self.push(c).map_err(|_| fmt::Error)
+        self.try_push(c).map_err(|_| fmt::Error)
     }
+
     fn write_str(&mut self, s: &str) -> fmt::Result {
-        self.push_str(s).map_err(|_| fmt::Error)
+        self.try_push_str(s).map_err(|_| fmt::Error)
     }
 }
 
@@ -281,10 +327,90 @@ impl<A: Array<Item=u8> + Copy> Clone for ArrayString<A> {
     fn clone(&self) -> ArrayString<A> {
         *self
     }
-
     fn clone_from(&mut self, rhs: &Self) {
         // guaranteed to fit due to types matching.
         self.clear();
-        self.push_str(rhs).ok();
+        self.try_push_str(rhs).ok();
+    }
+}
+
+impl<A: Array<Item=u8>> PartialOrd for ArrayString<A> {
+    fn partial_cmp(&self, rhs: &Self) -> Option<cmp::Ordering> {
+        (**self).partial_cmp(&**rhs)
+    }
+    fn lt(&self, rhs: &Self) -> bool { **self < **rhs }
+    fn le(&self, rhs: &Self) -> bool { **self <= **rhs }
+    fn gt(&self, rhs: &Self) -> bool { **self > **rhs }
+    fn ge(&self, rhs: &Self) -> bool { **self >= **rhs }
+}
+
+impl<A: Array<Item=u8>> PartialOrd<str> for ArrayString<A> {
+    fn partial_cmp(&self, rhs: &str) -> Option<cmp::Ordering> {
+        (**self).partial_cmp(rhs)
+    }
+    fn lt(&self, rhs: &str) -> bool { &**self < rhs }
+    fn le(&self, rhs: &str) -> bool { &**self <= rhs }
+    fn gt(&self, rhs: &str) -> bool { &**self > rhs }
+    fn ge(&self, rhs: &str) -> bool { &**self >= rhs }
+}
+
+impl<A: Array<Item=u8>> PartialOrd<ArrayString<A>> for str {
+    fn partial_cmp(&self, rhs: &ArrayString<A>) -> Option<cmp::Ordering> {
+        self.partial_cmp(&**rhs)
+    }
+    fn lt(&self, rhs: &ArrayString<A>) -> bool { self < &**rhs }
+    fn le(&self, rhs: &ArrayString<A>) -> bool { self <= &**rhs }
+    fn gt(&self, rhs: &ArrayString<A>) -> bool { self > &**rhs }
+    fn ge(&self, rhs: &ArrayString<A>) -> bool { self >= &**rhs }
+}
+
+impl<A: Array<Item=u8>> Ord for ArrayString<A> {
+    fn cmp(&self, rhs: &Self) -> cmp::Ordering {
+        (**self).cmp(&**rhs)
+    }
+}
+
+#[cfg(feature="serde-1")]
+impl<A: Array<Item=u8>> Serialize for ArrayString<A> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+        where S: Serializer
+    {
+        serializer.serialize_str(&*self)
+    }
+}
+
+#[cfg(feature="serde-1")]
+impl<'de, A: Array<Item=u8>> Deserialize<'de> for ArrayString<A> {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+        where D: Deserializer<'de>
+    {
+        use serde::de::{self, Visitor};
+        use std::marker::PhantomData;
+
+        struct ArrayStringVisitor<A: Array<Item=u8>>(PhantomData<A>);
+
+        impl<'de, A: Array<Item=u8>> Visitor<'de> for ArrayStringVisitor<A> {
+            type Value = ArrayString<A>;
+
+            fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+                write!(formatter, "a string no more than {} bytes long", A::capacity())
+            }
+
+            fn visit_str<E>(self, v: &str) -> Result<Self::Value, E>
+                where E: de::Error,
+            {
+                ArrayString::from(v).map_err(|_| E::invalid_length(v.len(), &self))
+            }
+
+            fn visit_bytes<E>(self, v: &[u8]) -> Result<Self::Value, E>
+                where E: de::Error,
+            {
+                let s = try!(str::from_utf8(v).map_err(|_| E::invalid_value(de::Unexpected::Bytes(v), &self)));
+
+                ArrayString::from(s).map_err(|_| E::invalid_length(s.len(), &self))
+            }
+        }
+
+        deserializer.deserialize_str(ArrayStringVisitor::<A>(PhantomData))
     }
 }
